@@ -747,7 +747,7 @@ static void emit_coverage(const unit_meta_t *meta, jit_t *j, cover_data_t *db)
       fatal_errno("failed to open coverage database: %s", meta->cover_file);
 
    const int n_tags = cover_count_items(db);
-   const int32_t *counts = jit_get_cover_mem(j, n_tags);
+   const int32_t *counts = jit_sync_cover_mem(j, n_tags);
    cover_dump_items(db, f, COV_DUMP_RUNTIME, counts);
 
    fbuf_close(f, NULL);
@@ -861,7 +861,7 @@ static int run_cmd(int argc, char **argv, cmd_state_t *state)
 
             // TODO: add an unconditional warning after 1.16
             if (state->jit != NULL && opt_get_int(OPT_IEEE_WARNINGS) != on)
-               warnf("the $bold$--ieee-warnings$$ option may have no affect "
+               warnf("the $bold$--ieee-warnings$$ option may have no effect "
                      "as the IEEE packages have already been initialised, pass "
                      "$bold$--ieee-warnings$$ as a global option instead");
 
